package VASL.build.module.map;

import VASSAL.build.AbstractToolbarItem;
import VASSAL.build.Buildable;
import VASSAL.build.GameModule;
import VASSAL.build.module.GameComponent;
import VASSAL.build.module.Map;
import VASSAL.build.module.documentation.HelpFile;
import VASSAL.build.module.map.PieceMover;
import VASSAL.build.widget.PieceSlot;
import VASSAL.counters.*;
import VASSAL.tools.LaunchButton;
import VASSAL.tools.imageop.ImageOp;
import VASSAL.tools.imageop.Op;
import VASSAL.tools.imageop.OpMultiResolutionImage;

import javax.swing.*;
import java.awt.*;
import java.awt.dnd.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Hashtable;

/**
 * Class to add a button to a map toolbar.
 * Pressing the button identifies any piece ot stack currently selected.
 **/
public class DCMenus extends AbstractToolbarItem
        implements PropertyChangeListener, GameComponent {

    protected static final String BUTTON_TEXT = "buttonText"; //NON-NLS
    public static final String MENU_PROPERTY = "ToolbarMenu.popup"; //$NON-NLS-1$
    public static final String MENU_ICON_PROPERTY = "ToolbarMenu.popupIcon"; //$NON-NLS
    public static final String HIDDEN_BY_TOOLBAR = "hidden"; //$NON-NLS-1$

    protected Map map;
    protected JToolBar toolbar;
    protected Runnable menuBuilder;
    protected java.util.List<String> submenuItems = new ArrayList<>();
    private Hashtable mar_HashPieceSlot = new Hashtable();

    protected final int buttonSize = 22;
    protected JPopupMenu menu;
    protected JMenu mAllied;
    protected JMenu mAxis;
    protected JMenu mPerimeter;
    protected JCheckBox useSmallSelect;
    protected boolean useSmall;
    protected final String[] menuList = {
            "Allied",
            "Axis",
            "Perimeter"
    };
    protected final String[] AlliedGPIDList = {
            "146:0",
            "146:48",
            "146:50",
            "146:52",
            "146:54",
            "146:56",
            "146:58",
            "146:60",
            "146:62",
            "146:64",
            "146:4",
            "146:5",
            "146:6",
            "146:7",
            "146:8",
            "146:9",
            "146:10",
            "146:11",
            "146:21"
    };
    protected final String[] AlliedGPIDListSmall = {
            "146:392",
            "146:394",
            "146:396",
            "146:398",
            "146:400",
            "146:402",
            "146:404",
            "146:406",
            "146:408",
            "146:410",
            "146:412",
            "146:413",
            "146:414",
            "146:415",
            "146:416",
            "146:417",
            "146:418",
            "146:419",
            "146:420"
    };
    protected final String[] AxisGPIDList = {
            "146:45",
            "146:66",
            "146:68",
            "146:70",
            "146:72",
            "146:74",
            "146:76",
            "146:78",
            "146:80",
            "146:82",
            "146:12",
            "146:13",
            "146:14",
            "146:15",
            "146:16",
            "146:17",
            "146:18",
            "146:19",
            "146:20",
            "146:22"
    };
    protected final String[] AxisGPIDListSmall = {
            "146:421",
            "146:423",
            "146:425",
            "146:427",
            "146:429",
            "146:431",
            "146:433",
            "146:435",
            "146:437",
            "146:439",
            "146:441",
            "146:442",
            "146:443",
            "146:444",
            "146:445",
            "146:446",
            "146:447",
            "146:448",
            "146:449",
            "146:450"
    };
    protected final String[] PerimeterGPIDList = {
            "146:94",
            "146:118",
            "146:122",
            "146:126",
            "146:130",
            "146:134",
            "146:138",
            "146:142",
            "146:451",
            "146:475",
            "146:477",
            "146:479",
            "146:481",
            "146:483",
            "146:485",
            "146:487"
    };

    public DCMenus() {
        setNameKey("");
        setButtonTextKey(BUTTON_TEXT);
        setShowDisabledOptions(false); //AbstractToolbarItem
        useSmall = false;
        setLaunchButton(makeLaunchButton(
                "tool tip",
                "button text",
                "",
                e -> apply()
        ));
        launch = getLaunchButton(); // for compatibility
        launch.setMargin(new Insets(0, 0, 0, 0));
        menu = new JPopupMenu();
        getLaunchButton().putClientProperty(MENU_PROPERTY, menu);
        getLaunchButton().putClientProperty(MENU_ICON_PROPERTY, launch.getLaunchIcon()); // In case we're a *submenu* of another ToolbarMenu, tell it about our beautiful icon
        GameModule.getGameModule().getGameState().addGameComponent(this);
    }

    private void apply() {
        final LaunchButton lb = getLaunchButton();
        if (lb.isShowing()) {
            menu.show(lb, 0, lb.getHeight());
            useSmallSelect.setEnabled(true);
            mAllied.setEnabled(true);
            mAxis.setEnabled(true);
            mPerimeter.setEnabled(true);
        }
    }

    protected void scheduleBuildMenu() {
        if (menuBuilder == null) {
            menuBuilder = () -> {
                buildMenu();
                menuBuilder = null;
            };
            SwingUtilities.invokeLater(menuBuilder);
        }
    }

    protected void buildMenu() {
        ReadPieceSlots();
        menu.removeAll();

        mAllied = new JMenu(menuList[0]);
        mAxis = new JMenu(menuList[1]);
        mPerimeter = new JMenu(menuList[2]);
        useSmallSelect = new JCheckBox("Small", useSmall);
        useSmallSelect.addItemListener(new ItemListener() {
            @Override
            public void itemStateChanged(ItemEvent e) {
                useSmall = useSmallSelect.isSelected();
                mAllied.setVisible(false);
                mAxis.setVisible(false);
                mPerimeter.setVisible(false);
                buildSubMenus();
                mAllied.setVisible(true);
                mAxis.setVisible(true);
                mPerimeter.setVisible(true);
            }
        });

        buildSubMenus();

        menu.add(useSmallSelect);
        menu.add(mAllied);
        menu.add(mAxis);
        menu.add(mPerimeter);
    }

    protected void buildSubMenus() {
        mAllied.removeAll();
        submenuItems = new ArrayList<>(Arrays.asList(useSmall ? AlliedGPIDListSmall : AlliedGPIDList));
        for (final String strGpID : submenuItems) {
            PieceSlot ps = (PieceSlot) mar_HashPieceSlot.get(strGpID);
            final DCMenuItem mi = new DCMenuItem(ps, mAllied.getPopupMenu());
            mi.InitDragDrop();
            mi.setIcon(new SizedImageIcon(CreateButtonIcon(ps), buttonSize, buttonSize));
            mi.addMouseListener(new MouseAdapter() { //Allow the counter to be dragged across the menu
                @Override
                public void mousePressed(MouseEvent e) {
                    useSmallSelect.setEnabled(false);
                    mAllied.setEnabled(false);
                    mAxis.setEnabled(false);
                    mPerimeter.setEnabled(false);
                    mi.setEnabled(true);
                }
            });
            mAllied.add(mi);
        }

        mAxis.removeAll();
        submenuItems = new ArrayList<>(Arrays.asList(useSmall ? AxisGPIDListSmall : AxisGPIDList));
        for (final String strGpID : submenuItems) {
            PieceSlot ps = (PieceSlot) mar_HashPieceSlot.get(strGpID);
            final DCMenuItem mi = new DCMenuItem(ps, mAxis.getPopupMenu());
            mi.InitDragDrop();
            mi.setIcon(new SizedImageIcon(CreateButtonIcon(ps), buttonSize, buttonSize));
            mi.addMouseListener(new MouseAdapter() { //Allow the counter to be dragged across the menu
                @Override
                public void mousePressed(MouseEvent e) {
                    useSmallSelect.setEnabled(false);
                    mAllied.setEnabled(false);
                    mAxis.setEnabled(false);
                    mPerimeter.setEnabled(false);
                    mi.setEnabled(true);
                }
            });
            mAxis.add(mi);
        }

        mPerimeter.removeAll();
        submenuItems = new ArrayList<>(Arrays.asList(PerimeterGPIDList));
        for (final String strGpID : submenuItems) {
            PieceSlot ps = (PieceSlot) mar_HashPieceSlot.get(strGpID);
            final DCMenuItem mi = new DCMenuItem(ps, mPerimeter.getPopupMenu());
            mi.InitDragDrop();
            mi.setIcon(new SizedImageIcon(CreateButtonIcon(ps), buttonSize, buttonSize));
            mi.addMouseListener(new MouseAdapter() { //Allow the counter to be dragged across the menu
                @Override
                public void mousePressed(MouseEvent e) {
                    useSmallSelect.setEnabled(false);
                    mAllied.setEnabled(false);
                    mAxis.setEnabled(false);
                    mPerimeter.setEnabled(false);
                    mi.setEnabled(true);
                }
            });
            mPerimeter.add(mi);
        }
    }

    private void ReadPieceSlots() {
        java.util.List<PieceSlot> lar_PieceSlotL = GameModule.getGameModule().getAllDescendantComponentsOf(PieceSlot.class);
        for (PieceSlot l_objPieceSlot : lar_PieceSlotL) {
            mar_HashPieceSlot.put(l_objPieceSlot.getGpId(), l_objPieceSlot);
        }
        lar_PieceSlotL = null;
    }

    public Image CreateButtonIcon(PieceSlot ps) {
        final GamePiece p = ps.getPiece();
        final ImageOp pop = Op.piece(p);
        final Dimension d = pop.getSize();
        final int min = Math.min(d.width, d.height);
        return new OpMultiResolutionImage(Op.crop(pop, 0, 0, min, min));
    }

    public static class SizedImageIcon extends ImageIcon {
        protected final int w;
        protected final int h;
        private final Image img;

        public SizedImageIcon(Image img, int w, int h) {
            super(img);
            this.w = w;
            this.h = h;
            this.img = img;
        }

        @Override
        public int getIconHeight() {
            return h;
        }

        @Override
        public int getIconWidth() {
            return w;
        }

        @Override
        public void paintIcon(Component c, Graphics g, int x, int y) {
            g.drawImage(img, x, y, w, h, c);
        }
    }

    @Override
    public void addTo(Buildable b) {
        map = (Map) b;
        toolbar = map.getToolBar();
        toolbar.add(getLaunchButton());
        scheduleBuildMenu();
        GameModule.getGameModule().getGameState().addGameComponent(this);
    }

    @Override
    public void removeFrom(Buildable b) {
        map = (Map) b;
        map.getToolBar().remove(getLaunchButton());
        map.getToolBar().revalidate();
        GameModule.getGameModule().getGameState().removeGameComponent(this);
    }

    @Override
    public HelpFile getHelpFile() {
        return null;
    }

    @Override
    public Class<?>[] getAllowableConfigureComponents() {
        return new Class<?>[0];
    }

    @Override
    public void setup(boolean gameStarting) {
        super.setup(gameStarting);
        // Prevent our Toolbar buttons from becoming visible on Game close/reopen
        scheduleBuildMenu();
    }

    private void OutputString(String strMsg) {
        GameModule.getGameModule().getChatter().send(strMsg);
    }

//}

    //Not used
    class DCButton extends JButton {
        VASSAL.build.widget.PieceSlot m_objPieceSlot;

        public DCButton(VASSAL.build.widget.PieceSlot objPieceSlot) {
            super();
            m_objPieceSlot = objPieceSlot;
        }

        public void InitDragDrop() {
            DragGestureListener dragGestureListener = new DragGestureListener() {
                public void dragGestureRecognized(DragGestureEvent dge) {
                    startDrag();
                    PieceMover.AbstractDragHandler.getTheDragHandler().dragGestureRecognized(dge);
                }
            };
            DragSource.getDefaultDragSource().createDefaultDragGestureRecognizer(this, DnDConstants.ACTION_MOVE, dragGestureListener);
        }

        // Puts counter in DragBuffer. Call when mouse gesture recognized
        protected void startDrag() {
            if (m_objPieceSlot != null) {
                m_objPieceSlot.getPiece().setPosition(new Point(0, 0));

                // Erase selection border to avoid leaving selected after mouse dragged out
                m_objPieceSlot.getPiece().setProperty(Properties.SELECTED, null);

                if (m_objPieceSlot.getPiece() != null) {
                    KeyBuffer.getBuffer().clear();
                    DragBuffer.getBuffer().clear();
                    GamePiece l_objNewPiece = PieceCloner.getInstance().clonePiece(m_objPieceSlot.getPiece());
                    l_objNewPiece.setProperty(Properties.PIECE_ID, m_objPieceSlot.getGpId());
                    DragBuffer.getBuffer().add(l_objNewPiece);
                }
            }
        }
    }

    class DCMenuItem extends JMenuItem implements DragSourceListener {
        VASSAL.build.widget.PieceSlot m_objPieceSlot;
        JPopupMenu m_objParentPopupMenu;

        public DCMenuItem(VASSAL.build.widget.PieceSlot objPieceSlot, JPopupMenu objPopupMenu) {
            super();
            m_objPieceSlot = objPieceSlot;
            m_objParentPopupMenu = objPopupMenu;
        }

        public DCMenuItem(PieceSlot objPieceSlot, JMenu objMenu) {
            super();
            m_objPieceSlot = objPieceSlot;
        }

        public void InitDragDrop() {
            DragGestureListener dragGestureListener = e -> {
                // FredKors 18.11.2015 Seems to be obsolete
                //if ((dge.getComponent() != null) && (dge.getComponent() instanceof DCMenuItem))
                //DragSource.getDefaultDragSource().addDragSourceListener(((DCMenuItem)dge.getComponent()));

                startDrag();
                PieceMover.AbstractDragHandler.getTheDragHandler().dragGestureRecognized(e);

                // FredKors 18.11.2015 Move the code after the dragGestureRecognized to avoid exception
                // FredKors 29.03.2015 Fix a small annoying thing dragging a counter from a menu
                if (m_objParentPopupMenu != null) {
                    m_objParentPopupMenu.setVisible(false);
                }
                menu.setVisible(false);
            };
            DragSource.getDefaultDragSource().createDefaultDragGestureRecognizer(this, DnDConstants.ACTION_MOVE, dragGestureListener);
        }

        // Puts counter in DragBuffer. Call when mouse gesture recognized
        protected void startDrag() {
            if (m_objPieceSlot != null) {
                m_objPieceSlot.getPiece().setPosition(new Point(0, 0));

                // Erase selection border to avoid leaving selected after mouse dragged out
                m_objPieceSlot.getPiece().setProperty(Properties.SELECTED, null);

                if (m_objPieceSlot.getPiece() != null) {
                    KeyBuffer.getBuffer().clear();
                    DragBuffer.getBuffer().clear();
                    GamePiece l_objNewPiece = PieceCloner.getInstance().clonePiece(m_objPieceSlot.getPiece());
                    l_objNewPiece.setProperty(Properties.PIECE_ID, m_objPieceSlot.getGpId());
                    DragBuffer.getBuffer().add(l_objNewPiece);
                }
                // FredKors 18.11.2015 Move the code after the dragGestureRecognized to avoid exception
                // FredKors 29.03.2015 Fix a small annoying thing dragging a counter from a menu
                //if (m_objParentPopupMenu != null)
                //m_objParentPopupMenu.setVisible(false);
            }
        }

        public void dragEnter(DragSourceDragEvent dsde) {
        }

        public void dragOver(DragSourceDragEvent dsde) {
        }

        public void dropActionChanged(DragSourceDragEvent dsde) {
        }

        public void dragExit(DragSourceEvent dse) {
        }

        public void dragDropEnd(DragSourceDropEvent dsde) {
            DragSource.getDefaultDragSource().removeDragSourceListener(this);
        }
    }

}