/*
 *
 * Copyright (c) 2005 by Rodney Kinney
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License (LGPL) as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, copies are available
 * at http://www.opensource.org.
 */
//package VASSAL.build.module;
package VASL.build.module.map;
import VASSAL.build.module.Map;

import VASSAL.build.GameModule;
//import VASSAL.build.module.map.MassKeyCommand;
import VASL.build.module.map.MassKeyCommandSelect;
import VASSAL.configure.GlobalCommandTargetConfigurer;
import VASSAL.configure.GlobalCommandTargetConfigurer;
import VASSAL.configure.VisibilityCondition;
import VASSAL.counters.CounterGlobalKeyCommand;
//import VASSAL.counters.GlobalCommand;
import VASL.build.module.map.GlobalCommandSelect;
//import VASSAL.counters.GlobalCommandTarget;
import VASL.build.module.map.GlobalCommandTargetSelect;
import VASSAL.i18n.Resources;
import VASSAL.script.expression.AuditTrail;

import java.util.List;

/**
 * This version of {@link MassKeyCommand} is added directly to a {@link VASSAL.build.GameModule} and applies to all maps.
 *
 * The "Global Key Command" functionality, as the term is used in Vassal Modules, is spread out over several classes internally:
 * {@link GlobalCommand} - primary functionality for sending commands to multiple pieces based on matching parameters
 * {@link VASSAL.build.module.GlobalKeyCommand}         - Global Key Commands from a Module window
 * {@link VASSAL.build.module.StartupGlobalKeyCommand}  - Global Key Commands from a Module "At Startup"
 * {@link VASSAL.build.module.map.MassKeyCommand}       - Global Key Commands from a specific Map window
 * {@link VASSAL.build.module.map.DeckGlobalKeyCommand} - Global Key Commands from a Deck
 * {@link CounterGlobalKeyCommand}                      - Global Key Commands from a Game Piece
 *
 * Other important classes:
 * {@link GlobalCommandTarget}           - "Fast Match" parameters
 * {@link GlobalCommandTargetConfigurer} - configurer for "Fast Match" parameters
 */
public class GlobalKeyCommandSelect extends MassKeyCommandSelect {

  public GlobalKeyCommandSelect() {
    super();
  }

  public GlobalKeyCommandSelect(MassKeyCommandSelect gkc) {
    super(gkc);
  }

  /**
   * @return Our type of Global Key Command (overrides the one from Mass Key Command). Affects what configurer options are shown.
   */
  @Override
  public GlobalCommandTargetSelect.GKCtype getGKCtype() {
    return GlobalCommandTargetSelect.GKCtype.MODULE;
  }

  /**
   * This version of Global Key Commands searches every map in the module
   */
  @Override
  public void apply() {
    // getFilter() will build the expression and update the audit trail
    final AuditTrail audit = AuditTrail.create(this, "", Resources.getString("Editor.MassKey.match"));
    final List<Map> l = Map.getMapList();
    GameModule.getGameModule().sendAndLog(globalCommand.apply(l.toArray(new Map[0]), getFilter(audit), target, audit));
  }

  // Hide 'This Map only' option
  @Override
  public VisibilityCondition getAttributeVisibility(String name) {
    if (SINGLE_MAP.equals(name)) {
      return () -> false;
    }
    else {
      return super.getAttributeVisibility(name);
    }
  }
}
