package VASL.build.module.map;

import static VASL.build.module.map.boardPicker.ASLBoard.DEFAULT_HEX_HEIGHT;
import static VASL.build.module.map.boardPicker.ASLBoard.DEFAULT_HEX_WIDTH;
import static java.lang.Math.round;

import VASSAL.build.GameModule;
import VASSAL.build.module.Map;
import VASSAL.build.module.map.MapShader;
import VASSAL.build.module.map.boardPicker.Board;
import VASSAL.counters.Decorator;
import VASSAL.counters.GamePiece;
import VASSAL.counters.Stack;

import java.awt.*;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area;
import java.util.Arrays;

public class HCAMapShader extends MapShader {

    public HCAMapShader() { super(); }
    private void OutputString(String strMsg) {
        GameModule.getGameModule().getChatter().send(strMsg);
    }
    private String takeouttrigger = "Perimeter_area";
    protected Area mapClip = null;

    @Override
    protected Area getShadeShape(Map map) {
        final Area myShape = type.equals(FG_TYPE) ?
                new Area() : new Area(getBoardClip());
        Arrays.stream(map.getPieces()).forEach(p -> checkPieceAdd(myShape, p));
        Arrays.stream(map.getPieces()).forEach(p -> checkPieceSubtract(myShape, p));
        final Area outside = getMapClip();
        myShape.subtract(outside);
        return myShape;
    }

    //Add matching shaded piece shapes to the shaded area
    protected void checkPieceAdd(Area area, GamePiece piece) {
        if (piece instanceof Stack) {
            final Stack s = (Stack) piece;
            s.asList().forEach(gamePiece -> checkPieceAdd(area, gamePiece));
        }
        else {
            final HCAMapShader.ShadedPiece shaded = (HCAMapShader.ShadedPiece) Decorator.getDecorator(piece, HCAMapShader.ShadedPiece.class);
            if (shaded != null) {
                final Area shape = shaded.getArea(this);
                if (shape != null) {
                    if (type.equals(FG_TYPE)) {
                        area.add(shape);
                    }
                    else {
                        area.subtract(shape);
                    }
                }
            }
        }
    }

    //Subtract any shaded piece hex from the shaded area
    protected void checkPieceSubtract(Area area, GamePiece piece) {
        if (piece instanceof Stack) {
            final Stack s = (Stack) piece;
            s.asList().forEach(gamePiece -> checkPieceSubtract(area, gamePiece));
        }
        else {
            final String pname = piece.getName();
            final boolean takeout = pname.contains(takeouttrigger);
            if (takeout) {
                final Point mp = piece.getPosition();
                //OutputString(piece.getName()+" "+mp.x+" "+mp.y);
                Area baseShape = getSingleHexShape(mp.x,mp.y);
                final Board b = map.findBoard(mp);
                if (b != null) {
                    double mag = b.getMagnification();
                    //Need to identify and multiply by 3.0 for deluxe boards, as these have mag = 1.0 (!!)
                    //Is there a better way to do this?
                    String[] dxBoards = {"a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "d1", "d2", "d3", "d4", "d5", "d6", "d7"}; //Beyond this, deluxe names include "dx"
                    boolean deluxe = false;
                    String bdName = b.getName();
                    deluxe = ((Arrays.asList(dxBoards).contains(bdName)) || (bdName.contains("dx")));
                    if (deluxe) {mag = mag*3.0;}
                    if (mag != 1.0) {
                        final AffineTransform t = AffineTransform.getTranslateInstance(mp.x, mp.y);
                        t.scale(mag, mag);
                        t.translate(-mp.x, -mp.y);
                        baseShape = baseShape.createTransformedArea(t);
                    }
                }
                area.subtract(baseShape);
            }
        }
    }

    protected Area getSingleHexShape(int centerX, int centerY) {
        Polygon poly = new Polygon();

        float x = (float) centerX;
        float y = (float) centerY;

        float x1,y1, x2,y2, x3,y3, x4, y4, x5, y5, x6, y6;

        float deltaX = (float) (DEFAULT_HEX_WIDTH);
        float deltaY = (float) (DEFAULT_HEX_HEIGHT);

        float r = 2.F * deltaX / 3.F;

        Point p1 = new Point();
        Point p2 = new Point();
        Point p3 = new Point();
        Point p4 = new Point();
        Point p5 = new Point();
        Point p6 = new Point();

        x1 = x - r;
        y1 = y;
        p1.setLocation(round(x1), round(y1));

        x2 = x - 0.5F * r;
        y2 = y - 0.5F * deltaY;
        p2.setLocation(round(x2), round(y2));

        x3 = x + 0.5F * r;
        y3 = y2;
        p3.setLocation(round(x3) + 1, round(y3));

        x4 = x + r;
        y4 = y;
        p4.setLocation(round(x4) + 1, round(y4));

        x5 = x3;
        y5 = y + 0.5F * deltaY;
        p5.setLocation(round(x5) + 1, round(y5) + 1);

        x6 = x2;
        y6 = y5;
        p6.setLocation(round(x6), round(y6) + 1);

        poly.addPoint(p1.x, p1.y);
        poly.addPoint(p2.x, p2.y);
        poly.addPoint(p3.x, p3.y);
        poly.addPoint(p4.x, p4.y);
        poly.addPoint(p5.x, p5.y);
        poly.addPoint(p6.x, p6.y);
        poly.addPoint(p1.x, p1.y);

        return new Area(poly);
    }

    protected void buildMapClip() {
        Rectangle mv = new Rectangle(getMap().getComponent().getMaximumSize());
        //OutputString("Map x: "+mv.x+" y: "+mv.y+" width: "+mv.width+" height: "+mv.height);
        mapClip = new Area(mv);
        map.getBoards().forEach(b -> {
            Rectangle bb = b.bounds();
            mapClip.subtract(new Area(bb));
            //OutputString("Board name: " + b.getName() + " x: " + bb.x + " y: " + bb.y + " width: " + bb.width + " height: " + bb.height);
        });
    }

    public Area getMapClip() {
        buildMapClip();
        return mapClip;
    }
}