package VASL.build.module.map;

import VASSAL.build.AbstractToolbarItem;
import VASSAL.build.Buildable;
import VASSAL.build.GameModule;
import VASSAL.build.module.Map;
import VASSAL.build.module.documentation.HelpFile;
import VASSAL.build.widget.PieceSlot;
import VASSAL.counters.*;

import java.awt.*;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Objects;

import static java.lang.Math.floor;
import static java.lang.Math.round;

/**
 * Class to add a button to a map toolbar.
 * Pressing the button identifies any piece ot stack currently selected.
 **/
public class HCRefresher extends AbstractToolbarItem {
    protected static final String BUTTON_TEXT = "buttonText"; //NON-NLS
    protected Map map;
    private Hashtable HashPieceSlot = new Hashtable();
    //private Hashtable HashPieceName = new Hashtable(); //Dangerous - could replace pieces not in the extension
    private Hashtable HashOldControl = new Hashtable();
    private Hashtable HashOldPerim = new Hashtable();
    private Hashtable HashNewPerimProgression = new Hashtable();

    public HCRefresher() {
        setNameKey("");
        setButtonTextKey(BUTTON_TEXT);
        setShowDisabledOptions(false); //AbstractToolbarItem
        setLaunchButton(makeLaunchButton(
                "tool tip",
                "button text",
                "",
                e -> apply()
        ));
        launch = getLaunchButton(); // for compatibility
        launch.setMargin(new Insets(0, 0, 0, 0));
    }

    @Override
    public void addTo(Buildable b) {
        map = (Map) b;
        map.getToolBar().add(getLaunchButton());
        GameModule.getGameModule().getGameState().addGameComponent(this);
    }

    @Override
    public void removeFrom(Buildable b) {
        map = (Map) b;
        map.getToolBar().remove(getLaunchButton());
        map.getToolBar().revalidate();
        GameModule.getGameModule().getGameState().removeGameComponent(this);
    }

    @Override
    public HelpFile getHelpFile() {
        return null;
    }

    @Override
    public Class<?>[] getAllowableConfigureComponents() {
        return new Class<?>[0];
    }
    
    private void apply() {
        ReadPieceSlots();
        for (final GamePiece gp : map.getAllPieces()) {
            if (gp instanceof Stack) {
                Stack s = (Stack) gp;
                for (int i = s.getPieceCount(); i > 0; i--) { //Top down in the stack
                    GamePiece sgp = s.getPieceAt(i - 1);
                    checkForReplacement(sgp);
                }
            }
            else {
                checkForReplacement(gp);
            }
        }
    }

    private void checkForReplacement(GamePiece gp) {
        if ((gp.getProperty("Layer") == "Control") || (gp.getProperty("Layer") == "Perimeter")) {
            Point mapPt = gp.getPosition();
            Point snapPt = map.snapTo(mapPt);
            String locationName = map.localizedLocationName(snapPt);
            boolean oldPerimFound = false;

            //Search on PIECE_ID
            PieceSlot ps = (PieceSlot) HashPieceSlot.get(gp.getProperty(Properties.PIECE_ID));
            if (ps == null) {
                ps = (PieceSlot) HashOldControl.get(gp.getProperty(Properties.PIECE_ID));
            }
            if (ps == null) {
                ps = (PieceSlot) HashOldPerim.get(gp.getProperty(Properties.PIECE_ID));
                oldPerimFound = (ps != null ? true : false);
            }
            //if (ps == null) {
            //    //Search on piece name
            //    ps = (PieceSlot) HashPieceName.get(gp.getProperty("BasicName"));
            //}
            if (ps != null) {
                GamePiece np = PieceCloner.getInstance().clonePiece(ps.getPiece());
                np.setProperty(Properties.PIECE_ID, ps.getGpId());
                if (oldPerimFound) {
                    //Match number of perimeter sides for older perimeter pieces
                    np = matchPerimSides(gp, np);
                }
                //map.placeAt(np, snapPt);
                map.placeAt(np, mapPt);
                matchOwner(gp, np); //Match owner of original piece
                matchSpin(gp, np); //Match rotation state of original piece
                matchLayers(gp, np); //Match layer states of original piece
                //map.removePiece(gp); //Fails for offboard pieces
                gp.keyEvent(javax.swing.KeyStroke.getKeyStroke(KeyEvent.VK_D, InputEvent.CTRL_DOWN_MASK)); //Delete command
            }
            else {
                OutputString("Piece " + gp.getProperty("BasicName") + " not updated: ID = " + gp.getProperty(Properties.PIECE_ID) + " (" + gp.getId() + "), Position = " + gp.getPosition() + ", Hex = " + locationName);
            }
        }
    }

    private void ReadPieceSlots() {
        java.util.List<PieceSlot> PieceSlotList = GameModule.getGameModule().getAllDescendantComponentsOf(PieceSlot.class);
        HashPieceSlot.clear();
        for (PieceSlot objPieceSlot : PieceSlotList) {
            HashPieceSlot.put(objPieceSlot.getGpId(), objPieceSlot);
            //HashPieceName.put(objPieceSlot.getName(), objPieceSlot);
            //OutputString("Found "+objPieceSlot.getGpId()+" "+objPieceSlot.getName());
        }
        OldToNewSlots();
    }

    private void OldToNewSlots() {
        //Fallback mapping of old to new piece slots
        HashOldControl.clear();
        HashOldPerim.clear();
        HashOldControl.put("146:0", HashPieceSlot.get("146:0")); //Allied control
        HashOldControl.put("146:2", HashPieceSlot.get("146:45")); //Axis control
        HashOldPerim.put("146:23", HashPieceSlot.get("146:94")); //Perimeter-red
        HashOldPerim.put("146:25", HashPieceSlot.get("146:118")); //Perimeter-orange
        HashOldPerim.put("146:26", HashPieceSlot.get("146:122")); //Perimeter-gold
        HashOldPerim.put("146:27", HashPieceSlot.get("146:126")); //Perimeter-flouro
        HashOldPerim.put("146:28", HashPieceSlot.get("146:130")); //Perimeter-green
        HashOldPerim.put("146:29", HashPieceSlot.get("146:134")); //Perimeter-cyan
        HashOldPerim.put("146:30", HashPieceSlot.get("146:138")); //Perimeter-blue
        HashOldPerim.put("146:31", HashPieceSlot.get("146:142")); //Perimeter-purple

        String [] strArrayRed = {"146:94", "146:96", "146:98", "146:100", "146:102", "146:104"};
        String [] strArrayOrange = {"146:118", "146:174", "146:167", "146:160", "146:153", "146:146"};
        String [] strArrayGold = {"146:122", "146:209", "146:202", "146:195", "146:188", "146:181"};
        String [] strArrayFlouro = {"146:126", "146:244", "146:237", "146:230", "146:223", "146:216"};
        String [] strArrayGreen = {"146:130", "146:279", "146:272", "146:265", "146:258", "146:251"};
        String [] strArrayCyan = {"146:134", "146:314", "146:307", "146:300", "146:293", "146:286"};
        String [] strArrayBlue = {"146:138", "146:349", "146:342", "146:335", "146:328", "146:321"};
        String [] strArrayPurple = {"146:142", "146:384", "146:377", "146:370", "146:363", "146:356"};
        HashNewPerimProgression.put("146:94", strArrayRed);
        HashNewPerimProgression.put("146:118", strArrayOrange);
        HashNewPerimProgression.put("146:122", strArrayGold);
        HashNewPerimProgression.put("146:126", strArrayFlouro);
        HashNewPerimProgression.put("146:130", strArrayGreen);
        HashNewPerimProgression.put("146:134", strArrayCyan);
        HashNewPerimProgression.put("146:138", strArrayBlue);
        HashNewPerimProgression.put("146:142", strArrayPurple);
    }

    private void matchSpin(GamePiece gp, GamePiece np) {
        int facingCount = 0;
        double angle = 0.0;
        double newAnglePerFacing = 60.0;
        GamePiece trait = gp;
        trait = ((Decorator) trait).getInner();
        while (trait instanceof Decorator) {
            if (trait instanceof FreeRotator) {
                //String facingString = ((Decorator) trait).myGetState(); //Angle index 0 to max-1
                //facingCount = Integer.parseInt(facingString);
                angle = ((FreeRotator) trait).getAngle(); //Angle in degrees
                break;
            }
            trait = ((Decorator) trait).getInner();
        }
        while (angle < 0) {angle += 360.0;}
        angle = angle % 360.0;
        facingCount = (int) floor(angle / newAnglePerFacing); //Old counters rotate 30 degrees per rotation, not 60 degrees
        //Rotate the new piece the required number of rotations
        for (int j = 1; j <= facingCount; j++) {
            //np.keyEvent(javax.swing.KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_DOWN_MASK)); //Rotate command cw
            np.keyEvent(javax.swing.KeyStroke.getKeyStroke(KeyEvent.VK_Z, InputEvent.CTRL_DOWN_MASK)); //Rotate command ccw
        }
    }

    private void matchLayers(GamePiece gp, GamePiece np) {
        //Set the level and nation layers to the matching values
        int levelCount = 0;
        int nationCount = 0;
        GamePiece trait = gp;
        trait = ((Decorator) trait).getInner();
        while (trait instanceof Decorator) {
            if (trait instanceof Embellishment) {
                String layerString = ((Decorator) trait).myGetType();
                if (layerString.contains("Level")) {
                    levelCount = Integer.parseInt(((Decorator) trait).myGetState());
                    for (int j = 1; j <= levelCount-1; j++) {
                        np.keyEvent(javax.swing.KeyStroke.getKeyStroke(KeyEvent.VK_L, InputEvent.CTRL_DOWN_MASK));
                    }
                }
                if (layerString.contains("Nation")) {
                    nationCount = Integer.parseInt(((Decorator) trait).myGetState());
                    for (int j = 1; j <= nationCount-1; j++) {
                        np.keyEvent(javax.swing.KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_DOWN_MASK));
                    }
                }
            }
            if (trait instanceof AreaOfEffect) {
                if (((Decorator) trait).myGetType().contains("Radius")) {
                    np.setProperty("Radius",gp.getProperty("Radius"));
                }
            }
            trait = ((Decorator) trait).getInner();
        }
    }

    private void matchOwner(GamePiece gp, GamePiece np) {
        if (gp.getProperty("Owner") != null) {
            np.setProperty("Owner", gp.getProperty("Owner"));
        }
    }

    private GamePiece matchPerimSides(GamePiece gp, GamePiece np) {
        int sideCount = 0;
        GamePiece trait = gp;
        trait = ((Decorator) trait).getInner();
        while (trait instanceof Decorator) {
            if (trait instanceof Embellishment) {
                //String sideString = ((Decorator) trait).myGetState();
                String sideString = gp.getName();
                sideCount = Integer.parseInt(sideString);
                break;
            }
            trait = ((Decorator) trait).getInner();
        }
        //OutputString("Old perimeter piece found "+sideCount);
        String[] pieceList = (String[]) HashNewPerimProgression.get(np.getProperty(Properties.PIECE_ID));
        String pieceID = pieceList[sideCount-1];
        PieceSlot ps = (PieceSlot) HashPieceSlot.get(pieceID);
        GamePiece pp = PieceCloner.getInstance().clonePiece(ps.getPiece());
        pp.setProperty(Properties.PIECE_ID, ps.getGpId());
        return pp;
    }

    private void OutputString(String strMsg) {
        GameModule.getGameModule().getChatter().send(strMsg);
    }

}